@@ -22,6 +22,8 @@ modules = [
     "8chan",
     "8muses",
     "adultempire",
+    "agnph",
+    "ao3",
     "architizer",
     "artstation",
     "aryion",
@@ -30,9 +32,13 @@ modules = [
     "behance",
     "blogger",
     "bluesky",
+    "boosty",
     "bunkr",
     "catbox",
     "chevereto",
+    "cien",
+    "civitai",
+    "cohost",
     "comicvine",
     "cyberdrop",
     "danbooru",
@@ -42,7 +48,6 @@ modules = [
     "e621",
     "erome",
     "exhentai",
-    "fallenangels",
     "fanbox",
     "fanleaks",
     "fantia",
@@ -62,6 +67,7 @@ modules = [
     "hentaifox",
     "hentaihand",
     "hentaihere",
+    "hentainexus",
     "hiperdex",
     "hitomi",
     "hotleak",
@@ -83,6 +89,7 @@ modules = [
     "keenspot",
     "kemonoparty",
     "khinsider",
+    "koharu",
     "komikcast",
     "lensdump",
     "lexica",
@@ -113,7 +120,6 @@ modules = [
     "paheal",
     "patreon",
     "philomena",
-    "photobucket",
     "photovogue",
     "picarto",
     "piczel",
@@ -128,7 +134,6 @@ modules = [
     "pornhub",
     "pornpics",
     "postmill",
-    "pururin",
     "reactor",
     "readcomiconline",
     "reddit",
@@ -136,6 +141,7 @@ modules = [
     "rule34us",
     "sankaku",
     "sankakucomplex",
+    "scrolller",
     "seiga",
     "senmanga",
     "sexcom",
@@ -194,7 +200,6 @@ modules = [
     "directlink",
     "recursive",
     "oauth",
-    "test",
     "ytdl",
     "generic",
 ]
