@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2021-2022 Mike Fährmann
+# Copyright 2021-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -20,36 +20,23 @@ class LolisafeExtractor(BaseExtractor):
 
 
 BASE_PATTERN = LolisafeExtractor.update({
-    "xbunkr": {
-        "root": "https://xbunkr.com",
-        "pattern": r"xbunkr\.com",
-    }
 })
 
 
 class LolisafeAlbumExtractor(LolisafeExtractor):
     subcategory = "album"
     pattern = BASE_PATTERN + "/a/([^/?#]+)"
-    test = (
-        ("https://xbunkr.com/a/TA0bu3F4", {
-            "pattern": r"https://media\.xbunkr\.com/[^.]+\.\w+",
-            "count": 861,
-            "keyword": {
-                "album_id": "TA0bu3F4",
-                "album_name": "Hannahowo Onlyfans Photos",
-            }
-        }),
-        ("https://xbunkr.com/a/GNQc2I5d"),
-    )
+    example = "https://xbunkr.com/a/ID"
 
     def __init__(self, match):
         LolisafeExtractor.__init__(self, match)
-        self.album_id = match.group(match.lastindex)
+        self.album_id = self.groups[-1]
 
+    def _init(self):
         domain = self.config("domain")
-        if domain is None or domain == "auto":
-            self.root = text.root_from_url(match.group(0))
-        else:
+        if domain == "auto":
+            self.root = text.root_from_url(self.url)
+        elif domain:
             self.root = text.ensure_http_scheme(domain)
 
     def items(self):
@@ -60,7 +47,15 @@ class LolisafeAlbumExtractor(LolisafeExtractor):
             url = file["file"]
             file.update(data)
             text.nameext_from_url(url, file)
-            file["name"], sep, file["id"] = file["filename"].rpartition("-")
+
+            if "name" in file:
+                name = file["name"]
+                file["name"] = name.rpartition(".")[0] or name
+                file["id"] = file["filename"].rpartition("-")[2]
+            else:
+                file["name"], sep, file["id"] = \
+                    file["filename"].rpartition("-")
+
             yield Message.Url, url, file
 
     def fetch_album(self, album_id):
