@@ -27,11 +27,18 @@ class PixivExtractor(Extractor):
     filename_fmt = "{id}_p{num}.{extension}"
     archive_fmt = "{id}{suffix}.{extension}"
     cookies_domain = None
+    sanity_url = "https://s.pximg.net/common/images/limit_sanity_level_360.png"
+    mypixiv_url = "https://s.pximg.net/common/images/limit_mypixiv_360.png"
 
     def _init(self):
         self.api = PixivAppAPI(self)
         self.load_ugoira = self.config("ugoira", True)
         self.max_posts = self.config("max-posts", 0)
+        self.sanity_workaround = self.config("sanity", True)
+        self.meta_user = self.config("metadata")
+        self.meta_bookmark = self.config("metadata-bookmark")
+        self.meta_comments = self.config("comments")
+        self.meta_captions = self.config("captions")
 
     def items(self):
         tags = self.config("tags", "japanese")
@@ -46,11 +53,7 @@ class PixivExtractor(Extractor):
             def transform_tags(work):
                 work["tags"] = [tag["name"] for tag in work["tags"]]
 
-        url_sanity = ("https://s.pximg.net/common/images"
-                      "/limit_sanity_level_360.png")
         ratings = {0: "General", 1: "R-18", 2: "R-18G"}
-        meta_user = self.config("metadata")
-        meta_bookmark = self.config("metadata-bookmark")
         metadata = self.metadata()
 
         works = self.works()
@@ -60,18 +63,26 @@ class PixivExtractor(Extractor):
             if not work["user"]["id"]:
                 continue
 
-            meta_single_page = work["meta_single_page"]
-            meta_pages = work["meta_pages"]
-            del work["meta_single_page"]
-            del work["image_urls"]
-            del work["meta_pages"]
+            files = self._extract_files(work)
 
-            if meta_user:
+            if self.meta_user:
                 work.update(self.api.user_detail(work["user"]["id"]))
-            if meta_bookmark and work["is_bookmarked"]:
+            if self.meta_comments:
+                if work["total_comments"]:
+                    work["comments"] = list(
+                        self.api.illust_comments(work["id"]))
+                else:
+                    work["comments"] = ()
+            if self.meta_bookmark and work["is_bookmarked"]:
                 detail = self.api.illust_bookmark_detail(work["id"])
                 work["tags_bookmark"] = [tag["name"] for tag in detail["tags"]
                                          if tag["is_registered"]]
+            if self.meta_captions and not work.get("caption") and \
+                    not work.get("_mypixiv") and not work.get("_ajax"):
+                body = self._request_ajax("/illust/" + str(work["id"]))
+                if body:
+                    work["caption"] = text.unescape(body["illustComment"])
+
             if transform_tags:
                 transform_tags(work)
             work["num"] = 0
@@ -81,41 +92,178 @@ class PixivExtractor(Extractor):
             work.update(metadata)
 
             yield Message.Directory, work
+            for work["num"], file in enumerate(files):
+                url = file["url"]
+                work.update(file)
+                work["date_url"] = self._date_from_url(url)
+                yield Message.Url, url, text.nameext_from_url(url, work)
 
-            if work["type"] == "ugoira":
-                if not self.load_ugoira:
-                    continue
+    def _extract_files(self, work):
+        meta_single_page = work["meta_single_page"]
+        meta_pages = work["meta_pages"]
+        del work["meta_single_page"]
+        del work["image_urls"]
+        del work["meta_pages"]
 
+        if work["type"] == "ugoira":
+            if self.load_ugoira:
                 try:
-                    ugoira = self.api.ugoira_metadata(work["id"])
-                except exception.StopExtraction as exc:
+                    return self._extract_ugoira(work)
+                except Exception as exc:
                     self.log.warning(
-                        "Unable to retrieve Ugoira metatdata (%s - %s)",
-                        work.get("id"), exc.message)
-                    continue
-
-                url = ugoira["zip_urls"]["medium"].replace(
-                    "_ugoira600x600", "_ugoira1920x1080")
-                work["frames"] = ugoira["frames"]
-                work["date_url"] = self._date_from_url(url)
-                work["_http_adjust_extension"] = False
-                yield Message.Url, url, text.nameext_from_url(url, work)
-
-            elif work["page_count"] == 1:
-                url = meta_single_page["original_image_url"]
-                if url == url_sanity:
-                    self.log.debug("Skipping 'sanity_level' warning (%s)",
-                                   work["id"])
-                    continue
-                work["date_url"] = self._date_from_url(url)
-                yield Message.Url, url, text.nameext_from_url(url, work)
+                        "%s: Unable to retrieve Ugoira metatdata (%s - %s)",
+                        work["id"], exc.__class__.__name__, exc)
 
+        elif work["page_count"] == 1:
+            url = meta_single_page["original_image_url"]
+            if url == self.sanity_url:
+                if self.sanity_workaround:
+                    self.log.warning("%s: 'sanity_level' warning", work["id"])
+                    body = self._request_ajax("/illust/" + str(work["id"]))
+                    return self._extract_ajax(work, body)
+                else:
+                    self.log.warning(
+                        "%s: Unable to download work ('sanity_level' warning)",
+                        work["id"])
+            elif url == self.mypixiv_url:
+                work["_mypixiv"] = True
+                self.log.warning("%s: 'My pixiv' locked", work["id"])
+                return ()
             else:
-                for work["num"], img in enumerate(meta_pages):
-                    url = img["image_urls"]["original"]
-                    work["date_url"] = self._date_from_url(url)
-                    work["suffix"] = "_p{:02}".format(work["num"])
-                    yield Message.Url, url, text.nameext_from_url(url, work)
+                return ({"url": url},)
+
+        else:
+            return [
+                {
+                    "url"   : img["image_urls"]["original"],
+                    "suffix": "_p{:02}".format(num),
+                }
+                for num, img in enumerate(meta_pages)
+            ]
+
+        return ()
+
+    def _extract_ugoira(self, work):
+        ugoira = self.api.ugoira_metadata(work["id"])
+        url = ugoira["zip_urls"]["medium"]
+        work["_ugoira_frame_data"] = work["frames"] = frames = ugoira["frames"]
+        work["date_url"] = self._date_from_url(url)
+        work["_http_adjust_extension"] = False
+
+        if self.load_ugoira == "original":
+            work["_ugoira_original"] = True
+            base, sep, _ = url.rpartition("_ugoira")
+            base = base.replace("/img-zip-ugoira/", "/img-original/", 1) + sep
+
+            for ext in ("jpg", "png", "gif"):
+                try:
+                    url = "{}0.{}".format(base, ext)
+                    self.request(url, method="HEAD")
+                    break
+                except exception.HttpError:
+                    pass
+            else:
+                self.log.warning(
+                    "Unable to find Ugoira frame URLs (%s)", work["id"])
+
+            return [
+                {
+                    "url": "{}{}.{}".format(base, num, ext),
+                    "suffix": "_p{:02}".format(num),
+                    "_ugoira_frame_index": num,
+                }
+                for num in range(len(frames))
+            ]
+        else:
+            work["_ugoira_original"] = False
+            url = url.replace("_ugoira600x600", "_ugoira1920x1080", 1)
+            return ({"url": url},)
+
+    def _request_ajax(self, endpoint):
+        url = "{}/ajax{}".format(self.root, endpoint)
+        try:
+            return self.request(url, headers=self.headers_web).json()["body"]
+        except Exception:
+            return None
+
+    def _extract_ajax(self, work, body):
+        work["_ajax"] = True
+        url = self._extract_ajax_url(body)
+        if not url:
+            return ()
+
+        for key_app, key_ajax in (
+            ("title"            , "illustTitle"),
+            ("image_urls"       , "urls"),
+            ("create_date"      , "createDate"),
+            ("width"            , "width"),
+            ("height"           , "height"),
+            ("sanity_level"     , "sl"),
+            ("total_view"       , "viewCount"),
+            ("total_comments"   , "commentCount"),
+            ("total_bookmarks"  , "bookmarkCount"),
+            ("restrict"         , "restrict"),
+            ("x_restrict"       , "xRestrict"),
+            ("illust_ai_type"   , "aiType"),
+            ("illust_book_style", "bookStyle"),
+        ):
+            work[key_app] = body[key_ajax]
+
+        work["user"] = {
+            "account"    : body["userAccount"],
+            "id"         : int(body["userId"]),
+            "is_followed": False,
+            "name"       : body["userName"],
+            "profile_image_urls": {},
+        }
+
+        work["tags"] = tags = []
+        for tag in body["tags"]["tags"]:
+            name = tag["tag"]
+            try:
+                translated_name = tag["translation"]["en"]
+            except Exception:
+                translated_name = None
+            tags.append({"name": name, "translated_name": translated_name})
+
+        work["caption"] = text.unescape(body["illustComment"])
+        work["page_count"] = count = body["pageCount"]
+        if count == 1:
+            return ({"url": url},)
+
+        base, _, ext = url.rpartition("_p0.")
+        return [
+            {
+                "url"   : "{}_p{}.{}".format(base, num, ext),
+                "suffix": "_p{:02}".format(num),
+            }
+            for num in range(count)
+        ]
+
+    def _extract_ajax_url(self, body):
+        try:
+            original = body["urls"]["original"]
+            if original:
+                return original
+        except Exception:
+            pass
+
+        try:
+            square1200 = body["userIllusts"][body["id"]]["url"]
+        except Exception:
+            return
+        parts = square1200.rpartition("_p0")[0].split("/")
+        del parts[3:5]
+        parts[3] = "img-original"
+        base = "/".join(parts)
+
+        for ext in ("jpg", "png", "gif"):
+            try:
+                url = "{}_p0.{}".format(base, ext)
+                self.request(url, method="HEAD")
+                return url
+            except exception.HttpError:
+                pass
 
     @staticmethod
     def _date_from_url(url, offset=timedelta(hours=9)):
@@ -185,15 +333,17 @@ class PixivUserExtractor(PixivExtractor):
 class PixivArtworksExtractor(PixivExtractor):
     """Extractor for artworks of a pixiv user"""
     subcategory = "artworks"
+    _warning = True
     pattern = (BASE_PATTERN + r"/(?:"
                r"(?:en/)?users/(\d+)/(?:artworks|illustrations|manga)"
                r"(?:/([^/?#]+))?/?(?:$|[?#])"
                r"|member_illust\.php\?id=(\d+)(?:&([^#]+))?)")
     example = "https://www.pixiv.net/en/users/12345/artworks"
 
-    def __init__(self, match):
-        PixivExtractor.__init__(self, match)
-        u1, t1, u2, t2 = match.groups()
+    def _init(self):
+        PixivExtractor._init(self)
+
+        u1, t1, u2, t2 = self.groups
         if t1:
             t1 = text.unquote(t1)
         elif t2:
@@ -201,6 +351,14 @@ class PixivArtworksExtractor(PixivExtractor):
         self.user_id = u1 or u2
         self.tag = t1 or t2
 
+        if self.sanity_workaround:
+            self.cookies_domain = d = ".pixiv.net"
+            self._init_cookies()
+            if self._warning and not self.cookies.get("PHPSESSID", domain=d):
+                PixivArtworksExtractor._warning = False
+                self.log.warning("No 'PHPSESSID' cookie set. Can detect only "
+                                 "non R-18 'sanity_level' works.")
+
     def metadata(self):
         if self.config("metadata"):
             self.api.user_detail(self.user_id)
@@ -209,6 +367,19 @@ class PixivArtworksExtractor(PixivExtractor):
     def works(self):
         works = self.api.user_illusts(self.user_id)
 
+        if self.sanity_workaround:
+            body = self._request_ajax(
+                "/user/{}/profile/all".format(self.user_id))
+            try:
+                ajax_ids = list(map(int, body["illusts"]))
+                ajax_ids.extend(map(int, body["manga"]))
+                ajax_ids.sort()
+            except Exception as exc:
+                self.log.warning("Unable to collect artwork IDs using AJAX "
+                                 "API (%s: %s)", exc.__class__.__name__, exc)
+            else:
+                works = self._extend_sanity(works, ajax_ids)
+
         if self.tag:
             tag = self.tag.lower()
             works = (
@@ -218,6 +389,35 @@ class PixivArtworksExtractor(PixivExtractor):
 
         return works
 
+    def _extend_sanity(self, works, ajax_ids):
+        user = {"id": 1}
+        index = len(ajax_ids) - 1
+
+        for work in works:
+            while index >= 0:
+                work_id = work["id"]
+                ajax_id = ajax_ids[index]
+
+                if ajax_id == work_id:
+                    index -= 1
+                    break
+
+                elif ajax_id > work_id:
+                    index -= 1
+                    self.log.debug("Inserting work %s", ajax_id)
+                    yield self._make_work(ajax_id, self.sanity_url, user)
+
+                else:  # ajax_id < work_id
+                    break
+
+            yield work
+
+        while index >= 0:
+            ajax_id = ajax_ids[index]
+            self.log.debug("Inserting work %s", ajax_id)
+            yield self._make_work(ajax_id, self.sanity_url, user)
+            index -= 1
+
 
 class PixivAvatarExtractor(PixivExtractor):
     """Extractor for pixiv avatars"""
@@ -227,12 +427,12 @@ class PixivAvatarExtractor(PixivExtractor):
     pattern = USER_PATTERN + r"/avatar"
     example = "https://www.pixiv.net/en/users/12345/avatar"
 
-    def __init__(self, match):
-        PixivExtractor.__init__(self, match)
-        self.user_id = match.group(1)
+    def _init(self):
+        PixivExtractor._init(self)
+        self.sanity_workaround = self.meta_comments = False
 
     def works(self):
-        user = self.api.user_detail(self.user_id)["user"]
+        user = self.api.user_detail(self.groups[0])["user"]
         url = user["profile_image_urls"]["medium"].replace("_170.", ".")
         return (self._make_work("avatar", url, user),)
 
@@ -245,12 +445,12 @@ class PixivBackgroundExtractor(PixivExtractor):
     pattern = USER_PATTERN + "/background"
     example = "https://www.pixiv.net/en/users/12345/background"
 
-    def __init__(self, match):
-        PixivExtractor.__init__(self, match)
-        self.user_id = match.group(1)
+    def _init(self):
+        PixivExtractor._init(self)
+        self.sanity_workaround = self.meta_comments = False
 
     def works(self):
-        detail = self.api.user_detail(self.user_id)
+        detail = self.api.user_detail(self.groups[0])
         url = detail["profile"]["background_image_url"]
         if not url:
             return ()
@@ -307,6 +507,22 @@ class PixivWorkExtractor(PixivExtractor):
         return works
 
 
+class PixivUnlistedExtractor(PixivExtractor):
+    """Extractor for a unlisted pixiv illustrations"""
+    subcategory = "unlisted"
+    pattern = BASE_PATTERN + r"/(?:en/)?artworks/unlisted/(\w+)"
+    example = "https://www.pixiv.net/en/artworks/unlisted/a1b2c3d4e5f6g7h8i9j0"
+
+    def _extract_files(self, work):
+        body = self._request_ajax("/illust/unlisted/" + work["id"])
+        work["id_unlisted"] = work["id"]
+        work["id"] = text.parse_int(body["illustId"])
+        return self._extract_ajax(work, body)
+
+    def works(self):
+        return ({"id": self.groups[0], "user": {"id": 1}},)
+
+
 class PixivFavoriteExtractor(PixivExtractor):
     """Extractor for all favorites/bookmarks of a pixiv user"""
     subcategory = "favorite"
@@ -420,7 +636,8 @@ class PixivRankingExtractor(PixivExtractor):
                 self.log.warning("invalid date '%s'", date)
                 date = None
         if not date:
-            date = (datetime.utcnow() - timedelta(days=1)).strftime("%Y-%m-%d")
+            now = util.datetime_utcnow()
+            date = (now - timedelta(days=1)).strftime("%Y-%m-%d")
         self.date = date
 
         return {"ranking": {
@@ -550,9 +767,6 @@ class PixivSeriesExtractor(PixivExtractor):
     directory_fmt = ("{category}", "{user[id]} {user[account]}",
                      "{series[id]} {series[title]}")
     filename_fmt = "{num_series:>03}_{id}_p{num}.{extension}"
-    cookies_domain = ".pixiv.net"
-    browser = "firefox"
-    tls12 = False
     pattern = BASE_PATTERN + r"/user/(\d+)/series/(\d+)"
     example = "https://www.pixiv.net/user/12345/series/12345"
 
@@ -561,40 +775,24 @@ class PixivSeriesExtractor(PixivExtractor):
         self.user_id, self.series_id = match.groups()
 
     def works(self):
-        url = self.root + "/ajax/series/" + self.series_id
-        params = {"p": 1}
-        headers = {
-            "Accept": "application/json",
-            "Referer": "{}/user/{}/series/{}".format(
-                self.root, self.user_id, self.series_id),
-            "Alt-Used": "www.pixiv.net",
-        }
+        series = None
 
-        while True:
-            data = self.request(url, params=params, headers=headers).json()
-            body = data["body"]
-            page = body["page"]
+        for work in self.api.illust_series(self.series_id):
+            if series is None:
+                series = self.api.data
+                series["total"] = num_series = series.pop("series_work_count")
+            else:
+                num_series -= 1
 
-            series = body["extraData"]["meta"]
-            series["id"] = self.series_id
-            series["total"] = page["total"]
-            series["title"] = text.extr(series["title"], '"', '"')
-
-            for info in page["series"]:
-                work = self.api.illust_detail(info["workId"])
-                work["num_series"] = info["order"]
-                work["series"] = series
-                yield work
-
-            if len(page["series"]) < 10:
-                return
-            params["p"] += 1
+            work["num_series"] = num_series
+            work["series"] = series
+            yield work
 
 
 class PixivNovelExtractor(PixivExtractor):
     """Extractor for pixiv novels"""
     subcategory = "novel"
-    request_interval = 1.0
+    request_interval = (0.5, 1.5)
     pattern = BASE_PATTERN + r"/n(?:ovel/show\.php\?id=|/)(\d+)"
     example = "https://www.pixiv.net/novel/show.php?id=12345"
 
@@ -616,9 +814,8 @@ class PixivNovelExtractor(PixivExtractor):
                 work["tags"] = [tag["name"] for tag in work["tags"]]
 
         ratings = {0: "General", 1: "R-18", 2: "R-18G"}
-        meta_user = self.config("metadata")
-        meta_bookmark = self.config("metadata-bookmark")
         embeds = self.config("embeds")
+        covers = self.config("covers")
 
         if embeds:
             headers = {
@@ -634,9 +831,15 @@ class PixivNovelExtractor(PixivExtractor):
         if self.max_posts:
             novels = itertools.islice(novels, self.max_posts)
         for novel in novels:
-            if meta_user:
+            if self.meta_user:
                 novel.update(self.api.user_detail(novel["user"]["id"]))
-            if meta_bookmark and novel["is_bookmarked"]:
+            if self.meta_comments:
+                if novel["total_comments"]:
+                    novel["comments"] = list(
+                        self.api.novel_comments(novel["id"]))
+                else:
+                    novel["comments"] = ()
+            if self.meta_bookmark and novel["is_bookmarked"]:
                 detail = self.api.novel_bookmark_detail(novel["id"])
                 novel["tags_bookmark"] = [tag["name"] for tag in detail["tags"]
                                           if tag["is_registered"]]
@@ -650,7 +853,7 @@ class PixivNovelExtractor(PixivExtractor):
             yield Message.Directory, novel
 
             try:
-                content = self.api.novel_text(novel["id"])["novel_text"]
+                content = self.api.novel_webview(novel["id"])["text"]
             except Exception:
                 self.log.warning("Unable to download novel %s", novel["id"])
                 continue
@@ -658,12 +861,25 @@ class PixivNovelExtractor(PixivExtractor):
             novel["extension"] = "txt"
             yield Message.Url, "text:" + content, novel
 
+            if covers:
+                path = novel["image_urls"]["large"].partition("/img/")[2]
+                url = ("https://i.pximg.net/novel-cover-original/img/" +
+                       path.rpartition(".")[0].replace("_master1200", ""))
+                novel["date_url"] = self._date_from_url(url)
+                novel["num"] += 1
+                novel["suffix"] = "_p{:02}".format(novel["num"])
+                novel["_fallback"] = (url + ".png",)
+                url_jpg = url + ".jpg"
+                text.nameext_from_url(url_jpg, novel)
+                yield Message.Url, url_jpg, novel
+                del novel["_fallback"]
+
             if embeds:
                 desktop = False
                 illusts = {}
 
                 for marker in text.extract_iter(content, "[", "]"):
-                    if marker.startswith("[jumpuri:If you would like to "):
+                    if marker.startswith("uploadedimage:"):
                         desktop = True
                     elif marker.startswith("pixivimage:"):
                         illusts[marker[11:].partition("-")[0]] = None
@@ -824,11 +1040,12 @@ class PixivAppAPI():
         self.username = extractor._get_auth_info()[0]
         self.user = None
 
+        extractor.headers_web = extractor.session.headers.copy()
         extractor.session.headers.update({
             "App-OS"        : "ios",
-            "App-OS-Version": "13.1.2",
-            "App-Version"   : "7.7.6",
-            "User-Agent"    : "PixivIOSApp/7.7.6 (iOS 13.1.2; iPhone11,8)",
+            "App-OS-Version": "16.7.2",
+            "App-Version"   : "7.19.1",
+            "User-Agent"    : "PixivIOSApp/7.19.1 (iOS 16.7.2; iPhone12,8)",
             "Referer"       : "https://app-api.pixiv.net/",
         })
 
@@ -864,7 +1081,7 @@ class PixivAppAPI():
             "get_secure_url": "1",
         }
 
-        time = datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S+00:00")
+        time = util.datetime_utcnow().strftime("%Y-%m-%dT%H:%M:%S+00:00")
         headers = {
             "X-Client-Time": time,
             "X-Client-Hash": hashlib.md5(
@@ -889,6 +1106,10 @@ class PixivAppAPI():
         return self._call(
             "/v2/illust/bookmark/detail", params)["bookmark_detail"]
 
+    def illust_comments(self, illust_id):
+        params = {"illust_id": illust_id}
+        return self._pagination("/v3/illust/comments", params, "comments")
+
     def illust_follow(self, restrict="all"):
         params = {"restrict": restrict}
         return self._pagination("/v2/illust/follow", params)
@@ -901,11 +1122,20 @@ class PixivAppAPI():
         params = {"illust_id": illust_id}
         return self._pagination("/v2/illust/related", params)
 
+    def illust_series(self, series_id, offset=0):
+        params = {"illust_series_id": series_id, "offset": offset}
+        return self._pagination("/v1/illust/series", params,
+                                key_data="illust_series_detail")
+
     def novel_bookmark_detail(self, novel_id):
         params = {"novel_id": novel_id}
         return self._call(
             "/v2/novel/bookmark/detail", params)["bookmark_detail"]
 
+    def novel_comments(self, novel_id):
+        params = {"novel_id": novel_id}
+        return self._pagination("/v1/novel/comments", params, "comments")
+
     def novel_detail(self, novel_id):
         params = {"novel_id": novel_id}
         return self._call("/v2/novel/detail", params)["novel"]
@@ -918,6 +1148,15 @@ class PixivAppAPI():
         params = {"novel_id": novel_id}
         return self._call("/v1/novel/text", params)
 
+    def novel_webview(self, novel_id):
+        params = {"id": novel_id, "viewer_version": "20221031_ai"}
+        return self._call(
+            "/webview/v2/novel", params, self._novel_webview_parse)
+
+    def _novel_webview_parse(self, response):
+        return util.json_loads(text.extr(
+            response.text, "novel: ", ",\n"))
+
     def search_illust(self, word, sort=None, target=None, duration=None,
                       date_start=None, date_end=None):
         params = {"word": word, "search_target": target,
@@ -962,13 +1201,17 @@ class PixivAppAPI():
         params = {"illust_id": illust_id}
         return self._call("/v1/ugoira/metadata", params)["ugoira_metadata"]
 
-    def _call(self, endpoint, params=None):
+    def _call(self, endpoint, params=None, parse=None):
         url = "https://app-api.pixiv.net" + endpoint
 
         while True:
             self.login()
             response = self.extractor.request(url, params=params, fatal=False)
-            data = response.json()
+
+            if parse:
+                data = parse(response)
+            else:
+                data = response.json()
 
             if "error" not in data:
                 return data
@@ -985,10 +1228,15 @@ class PixivAppAPI():
 
             raise exception.StopExtraction("API request failed: %s", error)
 
-    def _pagination(self, endpoint, params, key="illusts"):
+    def _pagination(self, endpoint, params,
+                    key_items="illusts", key_data=None):
         while True:
             data = self._call(endpoint, params)
-            yield from data[key]
+
+            if key_data:
+                self.data = data.get(key_data)
+                key_data = None
+            yield from data[key_items]
 
             if not data["next_url"]:
                 return
@@ -996,6 +1244,6 @@ class PixivAppAPI():
             params = text.parse_query(query)
 
 
-@cache(maxage=10*365*24*3600, keyarg=0)
+@cache(maxage=36500*86400, keyarg=0)
 def _refresh_token_cache(username):
     return None
